import {
  Entity,
  Column,
  Index,
  PrimaryGeneratedColumn,
  BaseEntity,
  OneToMany,
  ManyToOne,
} from "typeorm";
import { Poll } from "~entities/poll.js";

@Entity()
@Index(["guildId", "name"], { unique: true })
export class PollCategory extends BaseEntity {
  @PrimaryGeneratedColumn()
  id: number;
  @Column("bigint", { nullable: false })
  guildId: string;
  @Column("text", { nullable: false })
  name: string;
  @Column("bigint")
  channelId: string | null = null;

  @OneToMany(() => PollRole, (blog) => blog.category, { nullable: true })
  roles: PollRole[];

  @OneToMany(() => Poll, (poll) => poll.category, { nullable: true })
  polls: Poll[];

  constructor(data: Partial<PollCategory>) {
    super();
    if (data == null) return;

    for (const [k, v] of Object.entries(data)) {
      this[k] = v;
    }
  }
}

@Entity()
@Index(["category", "roleId"], { unique: true })
@Index(["poll", "roleId"], { unique: true })
export class PollRole extends BaseEntity {
  @PrimaryGeneratedColumn()
  id: number;
  @ManyToOne(() => PollCategory, (category) => category.roles, {
    nullable: true,
  })
  category: PollCategory | null = null;
  @ManyToOne(() => Poll, (poll) => poll.roles, { nullable: true })
  poll: Poll | null = null;
  @Column("text", { nullable: false })
  roleId: string;
  @Column("text", { nullable: false })
  type: "manager" | "voter" = "voter";

  constructor(data: Partial<PollRole>) {
    super();
    if (data == null) return;

    for (const [k, v] of Object.entries(data)) {
      this[k] = v;
    }
  }
}
