import {
  Entity,
  Column,
  Index,
  PrimaryGeneratedColumn,
  BaseEntity,
  OneToMany,
  ManyToOne,
  PrimaryColumn,
  JoinColumn,
} from "typeorm";
import { PollCategory, PollRole } from "~entities/category.js";

@Entity()
@Index(["guildId", "id"])
export class Poll extends BaseEntity {
  @PrimaryGeneratedColumn()
  id: number;
  @Column("bigint", { nullable: false })
  guildId: string;

  @ManyToOne(() => PollCategory, (category) => category.polls, {
    nullable: true,
  })
  category: PollCategory | null;
  @OneToMany(() => PollRole, (role) => role.poll, { nullable: true })
  roles: PollRole[];

  @Column("text", { nullable: false })
  name: string;
  @OneToMany(() => PollOption, (option) => option.poll)
  options: PollOption[];
  @OneToMany(() => Vote, (vote) => vote.poll)
  votes: Vote[];

  @Column("bigint", { nullable: false })
  channelId: string;
  @Column("bigint", { nullable: true })
  @Index({ unique: true })
  messageId: string | null;

  constructor(data: Partial<Poll>) {
    super();
    if (data == null) return;

    for (const [k, v] of Object.entries(data)) {
      this[k] = v;
    }
  }
}

@Entity()
export class PollOption extends BaseEntity {
  @PrimaryGeneratedColumn()
  id: number;
  @ManyToOne(() => Poll, (poll) => poll.options)
  poll: Poll;

  @Column("text", { nullable: false })
  name: string;

  @OneToMany(() => Vote, (vote) => vote.option)
  votes: Vote[];

  constructor(data: Partial<PollOption>) {
    super();
    if (data == null) return;

    for (const [k, v] of Object.entries(data)) {
      this[k] = v;
    }
  }
}

@Entity()
export class Vote extends BaseEntity {
  @PrimaryColumn("integer")
  optionId: number;
  @PrimaryColumn("bigint")
  userId: string;
  @ManyToOne(() => PollOption, (option) => option.votes)
  @JoinColumn({ name: "optionId" })
  option: PollOption;
  @ManyToOne(() => Poll, (poll) => poll.votes)
  poll: Poll;

  constructor(data: Partial<Vote>) {
    super();
    if (data == null) return;

    for (const [k, v] of Object.entries(data)) {
      this[k] = v;
    }
  }
}
